# Logo SPHELP
Write-Output @"
  SSS   PPPP   H   H  EEEEE  L     PPPP   
 S      P   P  H   H  E      L     P   P  
  SSS   PPPP   HHHHH  EEEE   L     PPPP   
     S  P      H   H  E      L     P      
 SSSS   P      H   H  EEEEE  LLLLL  P      
                                         
"@

# Define the interval and duration for monitoring
$interval = 5  # seconds
$duration = 5  # seconds (5 minutes)
$samples = $duration / $interval

# Initialize an array to store process data
$processData = @()

# Collect data
for ($i = 0; $i -lt $samples; $i++) {
    $timestamp = Get-Date
    $processes = Get-Process | Select-Object Id, ProcessName, CPU, WorkingSet64

    foreach ($process in $processes) {
        $processData += [pscustomobject]@{
            Timestamp   = $timestamp
            Id          = $process.Id
            ProcessName = $process.ProcessName
            CPU         = $process.CPU
            Memory      = [math]::round($process.WorkingSet64 / 1MB, 2)
        }
    }
    Start-Sleep -Seconds $interval
}

# Check if data was collected
if ($processData.Count -eq 0) {
    Write-Host "No process data collected." -ForegroundColor Red
    exit
}

# Summarize data to get top 20 consumers of memory and CPU
$topMemoryConsumers = $processData | Group-Object -Property ProcessName, Id | 
    Select-Object @{Name="ProcessName"; Expression={$_.Name}}, 
                  @{Name="TotalMemoryMB"; Expression={($_.Group | Measure-Object -Property Memory -Sum).Sum}} |
    Sort-Object -Property TotalMemoryMB -Descending | Select-Object -First 20

$topCpuConsumers = $processData | Group-Object -Property ProcessName, Id | 
    Select-Object @{Name="ProcessName"; Expression={$_.Name}}, 
                  @{Name="TotalCPU"; Expression={($_.Group | Measure-Object -Property CPU -Sum).Sum}} |
    Sort-Object -Property TotalCPU -Descending | Select-Object -First 20

# Format and display the results side by side
function Format-TableSideBySide {
    param (
        [Parameter(Mandatory=$true)]
        [string[]] $Header1,
        [Parameter(Mandatory=$true)]
        [string[]] $Data1,
        [Parameter(Mandatory=$true)]
        [string[]] $Header2,
        [Parameter(Mandatory=$true)]
        [string[]] $Data2
    )

    $width1 = ($Header1 + $Data1) | Measure-Object -Property Length -Maximum | Select-Object -ExpandProperty Maximum
    $width2 = ($Header2 + $Data2) | Measure-Object -Property Length -Maximum | Select-Object -ExpandProperty Maximum

    $formatString1 = "{0,-$width1}"
    $formatString2 = "{0,-$width2}"

    $separator = " | "
    $lineLength = $width1 + $width2 + $separator.Length

    Write-Host ($formatString1 -f $Header1[0]) + $separator + ($formatString2 -f $Header2[0])
    Write-Host ($formatString1 -f $Header1[1]) + $separator + ($formatString2 -f $Header2[1])
    Write-Host ("-" * $lineLength)

    for ($i = 0; $i -lt [math]::Max($Data1.Count, $Data2.Count); $i++) {
        $line1 = if ($i -lt $Data1.Count) { $Data1[$i] } else { "" }
        $line2 = if ($i -lt $Data2.Count) { $Data2[$i] } else { "" }
        Write-Host ($formatString1 -f $line1) + $separator + ($formatString2 -f $line2)
    }
}

$memoryHeader = @("Process Name", "Total Memory (MB)")
$cpuHeader = @("Process Name", "Total CPU Time (s)")

$memoryData = $topMemoryConsumers | ForEach-Object { "{0,-30} {1,15}" -f $_.ProcessName, $_.TotalMemoryMB }
$cpuData = $topCpuConsumers | ForEach-Object { "{0,-30} {1,15:N2}" -f $_.ProcessName, $_.TotalCPU }

Format-TableSideBySide -Header1 $memoryHeader -Data1 $memoryData -Header2 $cpuHeader -Data2 $cpuData
